@@ -1,20 +1,20 @@
 // Include files							/*{{{*/
-#include<config.h>
+#include <config.h>
 
 #include <apt-pkg/algorithms.h>
-#include <apt-pkg/upgrade.h>
-#include <apt-pkg/error.h>
+#include <apt-pkg/cacheset.h>
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/depcache.h>
+#include <apt-pkg/error.h>
 #include <apt-pkg/pkgcache.h>
-#include <apt-pkg/cacheset.h>
+#include <apt-pkg/upgrade.h>
 
-#include <apt-private/private-output.h>
 #include <apt-private/private-cachefile.h>
+#include <apt-private/private-output.h>
 
-#include <string.h>
-#include <ostream>
 #include <cstdlib>
+#include <cstring>
+#include <ostream>
 
 #include <apti18n.h>
 									/*}}}*/
@@ -22,7 +22,7 @@
 using namespace std;
 
 static bool SortPackagesByName(pkgCache * const Owner,
-      map_pointer_t const A, map_pointer_t const B)
+      map_pointer<pkgCache::Group> const A, map_pointer<pkgCache::Group> const B)
 {
    if (A == 0)
       return false;
@@ -42,11 +42,11 @@ void SortedPackageUniverse::LazyInit() const
       return;
    pkgCache * const Owner = data();
    // In Multi-Arch systems Grps are easier to sort than Pkgs
-   std::vector<map_pointer_t> GrpList;
+   std::vector<map_pointer<pkgCache::Group>> GrpList;
    List.reserve(Owner->Head().GroupCount);
    for (pkgCache::GrpIterator I{Owner->GrpBegin()}; I.end() != true; ++I)
       GrpList.emplace_back(I - Owner->GrpP);
-   std::stable_sort(GrpList.begin(), GrpList.end(), std::bind( &SortPackagesByName, Owner, std::placeholders::_1, std::placeholders::_2 ));
+   std::stable_sort(GrpList.begin(), GrpList.end(), [&](auto && l, auto && r) { return SortPackagesByName(Owner, l, r); });
    List.reserve(Owner->Head().PackageCount);
    for (auto G : GrpList)
    {
